# vim: set fileencoding=utf-8 :
#
# (C) 2016,2017 Johannes Hubertz <johannes@hubertz.de>
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, please see
#    <http://www.gnu.org/licenses/>
#
#    Additionally this program is licensed under the terms of the
#    Apache License, Version 2, see the top level directory right here,
#    or you can view there:  <http://www.apache.org/licenses/>
#
from __future__ import absolute_import, division, print_function

__all__ = [
    "__title__", "__author__", "__author_email__",
    "__license__", "__url__", "__version__",
]

__title__ = "iptables-converter"

__author__ = "Johannes Hubertz"
__author_email__ = "johannes@hubertz.de"

__license__ = "GNU General Public License version 3 or later, Apache License Version 2"

__url__ = "https://github.com/sl0/conv.git"

__version__ = "0.9.10"
